clc
clear
close all

load('filter.mat')

%Calculated impedance Rp=47kOhm, Cp=47pF
Y0 = 1/47e3 + 1i*2*pi*10e6*47e-12;
Z0 = 1/Y0;

%Normalised frequency
wn=0.0035*pi;
k=4;
N=round(k*2*pi/wn);

%Create independent variable n
n=0:N + length(filt.Coefficients);

%Create noisy deformed signal
u=sin(wn*n);% + 0.1*sin(2*wn*n) + 0.05*sin(3*wn*n) + 0.05*(rand(1,length(n))-0.5);
i=calculateCurrent(u,wn,10e6);

%Apply FIR filter
U=filter(filt,u);
I=filter(filt,i);

%Remove transient (work with last 2048 samples)
u=cropVariable(u,N);
i=cropVariable(i,N);
U=cropVariable(U,N);
I=cropVariable(I,N);
n=0:N-1;

plotUI(n,u,i);
plotUI(n,U,I);

%Evaluation by DFT
y=cos(n*wn)+1i*sin(n*wn);
U_dft=xcorr(u,y,0);
I_dft=xcorr(i,y,0);
Au=abs(U_dft)*2/N;
Ai=abs(I_dft)*2/N;
fprintf('Vyhodnocení impedance Rp=47k\x03a9, Cp=47pF\n');
fprintf('Výpočet:\tZ=%f, \x03c6=%f°\n',abs(Z0),rad2deg(angle(Z0)));
fprintf('FIR.\t\tZ=%f, \x03c6=%f°\n',rms(U)/rms(I),calculateAngle(U,I,k));
fprintf('DFT_def:\tZ=%f, \x03c6=%f°\n',Au/Ai,rad2deg(angle(U_dft)-angle(I_dft)));


%Goertzel algorythm
% U_g=goertzel(u,6);
% I_g=goertzel(i,6);
U_g=myGoertzel(u,k);
I_g=myGoertzel(i,k);
% [Q,I]=goertzelSTM(u,5,length(u));
% U_g = I + 1i*Q;
% [Q,I]=goertzelSTM(i,5,length(i));
% I_g = I + 1i*Q;
Au=abs(U_g)*2/N;
Ai=abs(I_g)*2/N;
phi=rad2deg(angle(U_g)-angle(I_g));
if phi>180
    phi=phi-360;
end
fprintf('DFT_goer:\tZ=%f, \x03c6=%f°\n',Au/Ai,phi);

function y=cropVariable(x,N)
    y=x(length(x)-N+1:end);
end

function i=calculateCurrent(u,wn,f)
    %Parallel capacity 47pF
    C=47e-12;
    %Parallel resistance 47kOhm
    R=47e3;
    
    dt= wn/(2*pi*f);
    
    i=zeros(1,length(u));    
    u=[0 u];
    for x=2:length(u)-1
        i(x) = u(x)/R + C*(u(x)-u(x-1))/dt;
    end
end

function plotUI(n,u,i)
    N=length(n);
    
    figure()
    
    yyaxis('left')
    plot(n,i*1e3)
    %ylim([-10 10])
    ylabel('Proud [mA]');
    
    yyaxis('right')
    plot(n,u)
    %ylim([-1.2 1.2])
    ylabel('Napětí [V]');      
    
    title('Průběh napětí a proudu')
    xlim([0 N-1])
    xticks([0 511:512:N-512 N-1])
    box on
    grid on
end

function phi=calculateAngle(u,i,k)
    [uZeros_rising uZeros_iPos]=getZeros(u);
    [iZeros_rising iZeros_iPos]=getZeros(i);
    
    n=length(uZeros_rising);
    if(mod(n,2)~=0)
        n=n-1;
    end
    
    phi=[];
    for i=1:2:k-1;
        if( (uZeros_rising(i)==iZeros_rising(i)) )
            phi(end+1)=360*( iZeros_iPos(i)-uZeros_iPos(i) + iZeros_iPos(i+1)-uZeros_iPos(i+1) ) / ( 2*(uZeros_iPos(i+2)-uZeros_iPos(i)));
        else
            phi(end+1)=inf;
        end
    end
    phi=mean(phi);
end

function [rising,iPos]=getZeros(x)
    rising=[];
    iPos=[];
    for i=2:length(x)
        if ( x(i-1) < 0 ) && ( x(i) > 0)
            rising(end+1)=true;
            dy = x(i)-x(i-1);
            iPos(end+1)= i-1 - x(i-1)/dy;            
        end
        if ( x(i-1) > 0 ) && ( x(i) < 0)
            rising(end+1)=false;
            dy = x(i)-x(i-1);
            iPos(end+1)= i-1 - x(i-1)/dy;
        end
    end
end

function y=myGoertzel(x,k)
    N=length(x);
    Ak=2*cos(2*pi*k/N);
    WN=exp(-1i*2*pi*k/N);
    s=zeros(1,3);
    for n=1:N-1
        s(1)=x(n) + Ak*s(2) - s(3);
        s(3)=s(2);
        s(2)=s(1);
    end
    s(1)=x(n) + Ak*s(2) - s(3);
    y=s(1) - WN*s(2);
end

function [I,Q] = goertzelSTM(x,k,N)
    w = -2*pi*k/N;
    cw = cos(w); c = 2*cw;
    sw = sin(w);
    z1=0; z2=0; % init
    for n = 1 : N
        z0 = x(n) + c*z1 - z2;
        z2 = z1;
        z1 = z0;
    end
%     y=z1*exp(1i*w);
%     I=real(y) - z2;
%     Q=imag(y);
    I = cw*z1 - z2;
    Q = sw*z1;
end