clc
clear
close all

%Volumetric humidity
theta=[0.33 1.9 3.5 6 8.7 13 17.7 22.15 26.9];
CpSFMU=[34.835 39.927 44.327 50.689 51.383 54.103 56.653 62.623 67.589];
CpAgilent=[23.163 29.763 35.133 44.487 44.547 51.770 54.843 68.077 74.167];

fitSFMU = fitlm(theta, CpSFMU);
PolySFMU = flip(fitSFMU.Coefficients.Estimate');
fitAgilent = fitlm(theta, CpAgilent);
PolyAgilent = flip(fitAgilent.Coefficients.Estimate');

x=linspace(0, 30, 1e3);
y1=polyval(PolySFMU,x);
y2=polyval(PolyAgilent,x);

figure();set(gcf,'position',[10,10,800,600]);
clf;
plot(theta, CpAgilent, 'x',theta, CpSFMU, 'x');
hold on;
set(gca,'ColorOrderIndex',1);
plot(x,y2,x,y1);

grid on
box on

title('Závislost kapacity na objemové vlhkosti půdy')
xlabel('Objemová vlhkost \Theta [%]')
ylabel('Kapacita sondy [pF]')

fitSFMU_text=sprintf("\ny=%.5fx %+.5f\nR^2=%f",PolySFMU(1),PolySFMU(2),fitSFMU.Rsquared.Ordinary);
fitAgilent_text=sprintf("\ny=%.5fx %+.5f\nR^2=%f",PolyAgilent(1),PolyAgilent(2),fitAgilent.Rsquared.Ordinary);
legend('Agilent 4287A','Testovaná měřicí jednotka',fitAgilent_text, fitSFMU_text,'location','northwest');

